#!/bin/sh

PASSFILE="${HOME}/.pgpass"

CHADO="chado"
CHADO_FUN="chadofun"

WORKDIR=$(dirname "$0")
DB_SOURCE="GnpIS"

CHADO_SQL_SCRIPT="${WORKDIR}/chado/sql/transplant_chado_extraction.sql"
CHADO_FUN_SQL_SCRIPT="${WORKDIR}/chadofun/sql/transplant_chado_functional_extraction.sql"

## Help display
###############

usage () {
    cat <<EOF

USAGE : 
		$0 -database <database_name> -db_type <${CHADO}|${CHADO_FUN}> -host <server_host> -port <database_port> -user <user> [-password <pass>] -url_base <url_base> [-output_file] -db_source <logical db name> [-v[v]]

DESCRIPTION : 
		Script used to extract data from genome (chado) database.

PARAMS:
       -database    the database to extract
       -db_type     the type of database to query (${CHADO} or ${CHADO_FUN})
       -url_base    the base of the URL used to construct valid links
       -user        the user to connect to the database
       -host        the host of PostgreSQL server
       -port        the port of PostgreSQL server
       -password    the password of given user
       -db_source   the logical database name from which data is extracted. Default: ${DB_SOURCE}
       -entry_type  the type of data used to fill entry_type Solr field.
       -output_file the output file into which data is extracted
       -v           display verbose informations
       -vv          display very verbose informations
       -h or --help print this help

WARNING:
		If not given, the credentials of the user MUST be referenced in ${PASSFILE}.

EOF
    exit 1
}

## Get commandline params
#########################

VERBOSE=0

# default values
HOST="shelob.versailles.inra.fr"
PORT=""
DATABASE=""
USER=""
URL_BASE=""
OUTPUT_FILE=""

# get params
while [ -n "$1" ]; do
    case $1 in
        -h) usage;shift 1;;
        --help) usage;shift 1;;
        -database) DATABASE=$2;shift 2;;
        -db_type) DB_TYPE=$2;shift 2;;
        -host) HOST=$2;shift 2;;
        -port) PORT=$2;shift 2;;
        -user) USER=$2; shift 2;;
        -password) export PGPASSWORD="\"$2\""; shift 2;;
        -url_base) URL_BASE=$2;shift 2;;
        -db_source) DB_SOURCE=$2;shift 2;;
        -entry_type) ENTRY_TYPE=$2;shift 2;;
        -output_file) OUTPUT_FILE=$2;shift 2;;
        -v) VERBOSE=1;shift 1;;
        -vv) VERBOSE=2;shift 1;;
        --) shift;break;;
        -*) echo && echo "Unknown option: $1" && echo;exit 1;;
        *) break;;
    esac
done

# check options
[ -z "${DATABASE}" ] && echo " -database is required" && usage && exit 1
[ -z "${HOST}" ] && echo " -host is required" && usage && exit 1
[ -z "${PORT}" ] && echo " -port is required" && usage && exit 1
[ -z "${USER}" ] && echo " -user is required" && usage && exit 1
[ -z "${URL_BASE}" ] && echo " -url_base is required" && usage && exit 1
[ -z "${DB_SOURCE}" ] && echo " -db_source requires a non empty value" && usage && exit 1
[ -z "$ENTRY_TYPE" ] && echo " -entry_type is required" && usage && exit 1

[ -z "$DB_TYPE" ] && echo " -db_type is required" && usage && exit 1
[ "${DB_TYPE}" != "${CHADO}" ] && [ "${DB_TYPE}" != "${CHADO_FUN}" ] && echo "Only ${CHADO} and ${CHADO_FUN} are allowed for -db_type option" && exit 1

# check output file
if [ -z "${OUTPUT_FILE}" ]; then
	OUTPUT_FILE="transplant_${DB_TYPE}_extraction_${DATABASE}.csv"
	echo "Using default output file: ${OUTPUT_FILE}"
fi

# check password file
if [ ! -f "${PASSFILE}" ]; then
    echo "The password file ($PASSFILE) does not exists!"
    exit 2
fi

# assign SQL script according to database type
if [ "${DB_TYPE}" = "${CHADO}" ]; then
  SQL_SCRIPT=${CHADO_SQL_SCRIPT}
elif [ "${DB_TYPE}" = "${CHADO_FUN}" ]; then
  SQL_SCRIPT=${CHADO_FUN_SQL_SCRIPT}
else
  echo "Bad db type: ${DB_TYPE}" && exit 1 
fi

# check sql file
if [ ! -f ${SQL_SCRIPT} ]; then
    echo "The SQL script (${SQL_SCRIPT}) does not exist!"
    exit 2
fi

echo "Extracting data from ${DATABASE} ${DB_TYPE} database into ${OUTPUT_FILE}..."

# display verbose informations such as connexion parameters
if [ $VERBOSE -ge 2 ]; then
  echo "\conninfo" | psql -U "${USER}" -h "${HOST}" -p "${PORT}" "${DATABASE}"
fi

CMD="psql -U ${USER} -h ${HOST} -p ${PORT} -f $SQL_SCRIPT ${DATABASE} -v url_base=\"${URL_BASE}\" -v db_name=${DATABASE} -v db_source=${DB_SOURCE} -v entry_type=\"${ENTRY_TYPE}\" -w > ${OUTPUT_FILE}"
[ $VERBOSE -ge 1 ] && echo -e "Executing command: \t${CMD}"
eval ${CMD}
if [ $? -ne 0 ];then
	echo "Warning: error occured, perhaps need a correctly ${PASSFILE} file?!"
	echo "Exiting."
	exit 1
fi

# remove two first lines (header + separating line added by psql) and 2 last lines (empty line + nb lines) all added by psql
[ $VERBOSE -ge 1 ] && echo "remove two first lines (header + separating line added by psql) and 2 last lines (empty line + nb lines) all added by psql"
tail -n +3 "${OUTPUT_FILE}" | head -n -2 > "${OUTPUT_FILE}".new && mv "${OUTPUT_FILE}".new "${OUTPUT_FILE}"

[ $VERBOSE -ge 1 ] && echo "replace all blank rounded pipes by commas and remove first space at line start"
sed -i "s/\s*|\s*/,/g;s/^\ //" "${OUTPUT_FILE}"

echo "Extraction done for ${DATABASE} database!"

export PGPASSWORD=""

exit 0
