#!/bin/sh

PASSFILE="${HOME}/.my.cnf"

BIO_DB_GFF="biodbgff"
BP_SEQ_FEATURE="bpseqfeature"

WORKDIR=$(dirname "$0")

BIO_DB_GFF_SQL_SCRIPT="${WORKDIR}/biodbgff/sql/transplant_biodbgff_extraction.sql"
BP_SEQ_FEATURE_SQL_SCRIPT="${WORKDIR}/bpseqfeature/sql/transplant_bpseqfeature_extraction.sql"

## Help display
###############

usage () {
    cat <<EOF

USAGE : 
		$0 -database <database_name> -db_type <biodbgff|bpseqfeature> -taxon <taxon> -url_base <url_base> -entry_type <type of data> [-host <server_host>] [-port <database_port>] [-user <user>] [-password <pass>] [-db_source <logical db name>] [-output_file] [-v[v]]

DESCRIPTION : 
		Script used to extract data from genome (BioDbGff or BpSeqFeatureStore) database.

PARAMS:
       -database    the database to extract
       -type        the type of database to query (biodbgff or bpseqfeature)
       -taxon       the taxon related to the data
       -url_base    the base of the URL used to construct valid links
       -db_source   the logical database name from which data is extracted. Default: ${DB_SOURCE}
       -entry_type  the type of data used to fill entry_type Solr field.
       -user        the user to connect to the database
       -host        the host of MySQL server
       -port        the port of MySQL server
       -password    the password of given user
       -output_file the output file into which data is extracted
       -v           display verbose informations
       -vv          display very verbose informations
       -h or --help print this help

WARNING:
		If not given, the credentials of the user MUST be referenced in ${PASSFILE} using the database name as a suffix. Example for a database named 'dbtest':

[clientdbtest]
user=dbtestuser
password=dbtestpassword

EOF
    exit 1
}

## Get commandline params
#########################

USE_SUFFIX=true
VERBOSE=0

# default values
HOST="shelob.versailles.inra.fr"
PORT="9101"
DB_TYPE=""
DB_SOURCE="GnpIS"
DATABASE=""
USER=""
TAXON=""
PASSWORD_OPTION=""
URL_BASE=""
OUTPUT_FILE=""

# get params
while [ -n "$1" ]; do
    case $1 in
        -h) usage;shift 1;;
        --help) usage;shift 1;;
        -database) DATABASE=$2;shift 2;;
        -db_type) DB_TYPE=$2;shift 2;;
        -db_source) DB_SOURCE=$2;shift 2;;
        -host) HOST=$2;shift 2;;
        -port) PORT=$2;shift 2;;
        -user) USER=$2; USE_SUFFIX=false; shift 2;;
        -taxon) TAXON=$2;shift 2;;
        -password) PASSWORD_OPTION="--password=\"$2\""; shift 2;;
        -url_base) URL_BASE=$2;shift 2;;
        -entry_type) ENTRY_TYPE=$2;shift 2;;
        -output_file) OUTPUT_FILE=$2;shift 2;;
        -v) VERBOSE=1;shift 1;;
        -vv) VERBOSE=2;shift 1;;
        --) shift;break;;
        -*) echo && echo "Unknown option: $1" && echo;exit 1;;
        *) break;;
    esac
done

# check options
[ -z "$DATABASE" ] && echo " -database is required" && usage && exit 1
[ -z "$DB_TYPE" ] && echo " -db_type is required" && usage && exit 1
[ -z "$ENTRY_TYPE" ] && echo " -entry_type is required" && usage && exit 1
[ "${DB_TYPE}" != "${BIO_DB_GFF}" ] && [ "${DB_TYPE}" != "${BP_SEQ_FEATURE}" ] && echo "Only ${BIO_DB_GFF} and ${BP_SEQ_FEATURE} are allowed for -db_type option" && exit 1
#[ -z "$HOST" ] && echo "-host is required" && usage && exit 1
#[ -z "$PORT" ] && echo "-port is required" && usage && exit 1
#[ -z "$USER" ] && echo "-user is required" && usage && exit 1
[ -z "$TAXON" ] && echo " -taxon is required" && usage && exit 1
[ -z "$URL_BASE" ] && echo " -url_base is required" && usage && exit 1

if [ -z "${OUTPUT_FILE}" ]; then
  OUTPUT_FILE="transplant_${DB_TYPE}_extraction_${DATABASE}.csv"
  echo "Using default output file: ${OUTPUT_FILE}"
fi

# check password file
if [ ! -f "${PASSFILE}" ]; then
    echo "The password file ($PASSFILE) does not exists!"
    exit 2
fi

# assign SQL script according to database type
if [ "${DB_TYPE}" = "${BIO_DB_GFF}" ]; then
  SQL_SCRIPT=${BIO_DB_GFF_SQL_SCRIPT}
elif [ "${DB_TYPE}" = "${BP_SEQ_FEATURE}" ]; then
  SQL_SCRIPT=${BP_SEQ_FEATURE_SQL_SCRIPT}
else
  echo "Bad db type: ${DB_TYPE}" && exit 1 
fi

# check sql file
if [ ! -f "${SQL_SCRIPT}" ]; then
    echo "The SQL script (${SQL_SCRIPT}) does not exist!"
    exit 2
fi

echo "Extracting data from ${DATABASE} ${DB_TYPE} database into ${OUTPUT_FILE}..."

# display verbose informations such as connexion parameters
if [ $VERBOSE -ge 2 ]; then
  mysql --defaults-group-suffix="${DATABASE}" -A -e 'status;' | grep -E 'user|Connection:|port'
fi
# prepare mysql command to execute according to the type of connexion used for setting user & password (using my.cnf or via command line arguments)
if [ $USE_SUFFIX = true ]; then
  cmd_connexion="mysql --defaults-group-suffix=${DATABASE} ${DATABASE}"
else
  cmd_connexion="mysql -h ${HOST} -u ${USER} ${PASSWORD_OPTION} -P ${PORT} ${DATABASE}"
fi
[ $VERBOSE -ge 2 ] && echo "Sql script is : ${SQL_SCRIPT}"

if [ "${DB_TYPE}" = "${BIO_DB_GFF}" ]; then
cmd="${cmd_connexion}\
 --compress -A -e 'set @taxon=\"${TAXON}\"; set @dbsource=\"${DB_SOURCE}\"; set @url_base=\"${URL_BASE}\"; set @dbname=\"${DATABASE}\"; set @entry_type=\"${ENTRY_TYPE}\"; source ${SQL_SCRIPT};' | tail -n +2 | sed -r \"s/\t/,/g;s/^'/\\\"/g;s/'$/\\\"/g;s/,'/,\\\"/g;s/',/\\\",/g\" | gzip -c > '${OUTPUT_FILE}.gz'"
elif [ "${DB_TYPE}" = "${BP_SEQ_FEATURE}" ]; then
cmd="${cmd_connexion}\
 --compress -A -e 'set @taxon=\"${TAXON}\"; set @dbsource=\"${DB_SOURCE}\"; set @url_base=\"${URL_BASE}\"; set @dbname=\"${DATABASE}\"; set @entry_type=\"${ENTRY_TYPE}\"; source ${SQL_SCRIPT};' | tail -n +2 | sed -r \"s/\t/,/g;s/\\\\\\\',/\' ,/g;s/'(\w+)\:\w+'/'\1'/g;s/^'/\\\"/g;s/'$/\\\"/g;s/,'/,\\\"/g;s/',/\\\",/g\" | gzip -c > '${OUTPUT_FILE}.gz'"
else
  echo "Bad db type: ${DB_TYPE}"
  exit 1
fi


[ $VERBOSE -ge 1 ] && echo -e "Going to exec command: \n\t${cmd}"

# Execute command to extract data
eval $cmd
if [ $? -ne 0 ];then
  echo "Error occured: should check that passfile is given and correclty filled (user, password for given database, etc.): ${PASSFILE}!"
  echo "Exiting."
  exit 1
fi

# [ $VERBOSE -ge 1 ] && echo "Remove first lines (header added by mysql)."
# tail -n +2 "${OUTPUT_FILE}" > "${OUTPUT_FILE}".new && mv "${OUTPUT_FILE}".new "${OUTPUT_FILE}"

# [ $VERBOSE -ge 1 ] && echo "Change separating tabulations by commas." # BOTH BIO_DB_GFF AND BP_SEQ_FEATURE
# sed -ri "s/\t/,/g" ${OUTPUT_FILE}

# [ $VERBOSE -ge 1 ] && echo "Escape 3' and 5' stuff." # BP_SEQ_FEATURE ONLY
# sed -ri "s/\\\\\\\',/\' ,/g" ${OUTPUT_FILE}

# [ $VERBOSE -ge 1 ] && echo "Remove source from feature_type field, ie: 'scaffold:GDEC' => 'scaffold'." # BP_SEQ_FEATURE ONLY
# sed -ri "s/'(\w+)\:\w+'/'\1'/g" ${OUTPUT_FILE}

# [ $VERBOSE -ge 1 ] && echo "Replace all delimiting simple quotes by double quotes..." # BOTH BIO_DB_GFF AND BP_SEQ_FEATURE

#   [ $VERBOSE -ge 2 ] && echo -e "\t- line start" # BOTH BIO_DB_GFF AND BP_SEQ_FEATURE
#   sed -i "s/^'/\"/g" ${OUTPUT_FILE} # line start

#   [ $VERBOSE -ge 2 ] && echo -e "\t- line end" # BOTH BIO_DB_GFF AND BP_SEQ_FEATURE
#   sed -i "s/'$/\"/g" ${OUTPUT_FILE} # line end

#   [ $VERBOSE -ge 2 ] && echo -e "\t- after comma" # BOTH BIO_DB_GFF AND BP_SEQ_FEATURE
#   sed -i "s/,'/,\"/g" ${OUTPUT_FILE} # after comma

#   [ $VERBOSE -ge 2 ] && echo -e "\t- before comma" # BOTH BIO_DB_GFF AND BP_SEQ_FEATURE
#   sed -i "s/',/\",/g" ${OUTPUT_FILE} # before comma

# using below one line sed command with multiple replacements (same treatments than commented above ones)
# saves time by 50% with a SSD, let's imagine what if used a classical HDD... o/
# if [ "${DB_TYPE}" = "${BIO_DB_GFF}" ]; then
#   sed -i -r "s/\t/,/g;s/^'/\"/g;s/'$/\"/g;s/,'/,\"/g;s/',/\",/g" "${OUTPUT_FILE}"
# elif [ "${DB_TYPE}" = "${BP_SEQ_FEATURE}" ]; then
#   sed -i -r "s/\t/,/g;s/\\\\\\\',/\' ,/g;s/'(\w+)\:\w+'/'\1'/g;s/^'/\"/g;s/'$/\"/g;s/,'/,\"/g;s/',/\",/g" "${OUTPUT_FILE}"
# else
#   echo "Bad db type: ${DB_TYPE}"
# fi

echo "Finished extraction for ${DATABASE} database!"
echo

exit 0
